@@ -57,18 +57,28 @@ import java.util.Map.Entry;
 import java.util.Set;
 import java.util.concurrent.TimeUnit;
 import org.apache.commons.logging.Log;
+import com.hazelcast.core.EntryEvent;
+import com.hazelcast.core.EntryListener;
 import com.hazelcast.core.HazelcastInstance;
 import com.hazelcast.core.IMap;
+import com.hazelcast.core.Member;
 import com.hazelcast.core.MultiMap;
 import com.hazelcast.core.Transaction;
 import com.openexchange.exception.OXException;
+import com.openexchange.java.ConcurrentSet;
+import com.openexchange.management.ManagementAware;
+import com.openexchange.management.ManagementObject;
 import com.openexchange.realtime.directory.DefaultResourceDirectory;
 import com.openexchange.realtime.directory.Resource;
+import com.openexchange.realtime.hazelcast.Services;
 import com.openexchange.realtime.hazelcast.channel.HazelcastAccess;
+import com.openexchange.realtime.hazelcast.management.HazelcastResourceDirectoryMBean;
+import com.openexchange.realtime.hazelcast.management.HazelcastResourceDirectoryManagement;
 import com.openexchange.realtime.packet.ID;
 import com.openexchange.realtime.packet.IDEventHandler;
 import com.openexchange.realtime.packet.Presence;
 import com.openexchange.realtime.util.IDMap;
+import com.openexchange.timer.TimerService;
 
 /**
  * {@link HazelcastResourceDirectory} - Keeps mappings of general {@link ID}s to full {@link ID}s and full {@link ID}s to {@link Resource}.
@@ -78,35 +88,94 @@ import com.openexchange.realtime.util.IDMap;
  * @author <a href="mailto:steffen.templin@open-xchange.com">Steffen Templin</a>
  * @author <a href="mailto:marc.arens@open-xchange.com">Marc Arens</a>
  */
-public class HazelcastResourceDirectory extends DefaultResourceDirectory {
+public class HazelcastResourceDirectory extends DefaultResourceDirectory implements ManagementAware<HazelcastResourceDirectoryMBean> {
+
+    /** The logger */
+    static final Log LOG = com.openexchange.log.Log.loggerFor(HazelcastResourceDirectory.class);
 
-    private static final Log LOG = com.openexchange.log.Log.loggerFor(HazelcastResourceDirectory.class);
-    
     /** Mapping of general IDs to full IDs e.g marc.arens@premium <-> ox://marc.arens@premuim/random. */
     private final String id_map;
 
     /** Mapping of full IDs to the Resource e.g. ox://marc.arens@premuim/random <-> ResourceMap */
     private final String resource_map;
 
+    private final HazelcastResourceDirectoryManagement managementObject;
+
+    /** Keep track of synthetic ids */
+    private final Set<ID> syntheticIDs;
+
     /**
      * Initializes a new {@link HazelcastResourceDirectory}.
+     * 
      * @param id_map the name of the apping of general IDs to full IDs e.g marc.arens@premium <-> ox://marc.arens@premuim/random
      * @param resource_map the name of the mapping of full IDs to the Resource e.g. ox://marc.arens@premuim/random <-> ResourceMap
+     * @throws OXException
      */
-    public HazelcastResourceDirectory(String id_map, String resource_map) {
+    public HazelcastResourceDirectory(String id_map, String resource_map) throws OXException {
         super();
         this.id_map = id_map;
         this.resource_map = resource_map;
+        this.managementObject = new HazelcastResourceDirectoryManagement(this);
+        syntheticIDs = new ConcurrentSet<ID>();
+        getResourceMapping().addEntryListener(new EntryListener<String, Map<String, Serializable>>() {
+
+            @Override
+            public void entryUpdated(EntryEvent<String, Map<String, Serializable>> event) { /* nothing */
+            }
+
+            @Override
+            public void entryRemoved(EntryEvent<String, Map<String, Serializable>> event) {
+                ID id = new ID(event.getKey());
+                boolean removed = syntheticIDs.remove(id);
+                if (LOG.isDebugEnabled()) {
+                    if (removed) {
+                        LOG.debug("Removed id from refresh list: " + id);
+                    }
+                }
+            }
+
+            @Override
+            public void entryAdded(EntryEvent<String, Map<String, Serializable>> event) { /* nothing */
+            }
+
+            @Override
+            public void entryEvicted(EntryEvent<String, Map<String, Serializable>> event) {
+                String id = event.getKey();
+                Object source = event.getSource();
+                Member member = event.getMember();
+                try {
+                    if (getIDMapping().remove(new ID(id).toGeneralForm().toString(), id)) {
+                        if (LOG.isDebugEnabled()) {
+                            LOG.debug("Source " + source + " on Member: " + member + " fired event. " + "Removing mapping for '" + id + "' due to eviction of according resource.");
+                        }
+                    }
+                } catch (OXException e) {
+                    LOG.warn("Could not handle eviction for id '" + id + "'", e);
+                }
+                boolean removed = syntheticIDs.remove(new ID(id));
+                if (LOG.isDebugEnabled()) {
+                    if (removed) {
+                        LOG.debug("Removed id from refresh list: " + id);
+                    }
+                }
+            }
+        }, false);
+        startRefreshTimer();
+    }
+
+    @Override
+    public ManagementObject<HazelcastResourceDirectoryMBean> getManagementObject() {
+        return managementObject;
     }
 
     @Override
     public IDMap<Resource> get(ID id) throws OXException {
-         IDMap<Resource> foundResources = new IDMap<Resource>();
+        IDMap<Resource> foundResources = new IDMap<Resource>();
         if (id.isGeneralForm()) {
             MultiMap<String, String> idMapping = getIDMapping();
             Collection<String> concreteIds = idMapping.get(id.toString());
             if (concreteIds != null && !concreteIds.isEmpty()) {
-                IMap<String, Map<String,Serializable>> allResources = getResourceMapping();
+                IMap<String, Map<String, Serializable>> allResources = getResourceMapping();
                 Map<String, Map<String, Serializable>> resources = allResources.getAll(new HashSet<String>(concreteIds));
                 if (resources != null) {
                     for (Entry<String, Map<String, Serializable>> entry : resources.entrySet()) {
@@ -117,7 +186,7 @@ public class HazelcastResourceDirectory extends DefaultResourceDirectory {
                 }
             }
         } else {
-            IMap<String,Map<String,Serializable>> allResources = getResourceMapping();
+            IMap<String, Map<String, Serializable>> allResources = getResourceMapping();
             Map<String, Serializable> resourceMap = allResources.get(id.toString());
             HazelcastResource resource = HazelcastResourceWrapper.unwrap(resourceMap);
             if (resource != null) {
@@ -147,7 +216,7 @@ public class HazelcastResourceDirectory extends DefaultResourceDirectory {
         }
 
         if (!resourceIds.isEmpty()) {
-            IMap<String,Map<String,Serializable>> allResources = getResourceMapping();
+            IMap<String, Map<String, Serializable>> allResources = getResourceMapping();
             Map<String, Map<String, Serializable>> matchingResources = allResources.getAll(IDWrapper.idsToStringSet(resourceIds));
             if (matchingResources != null) {
                 for (Entry<String, Map<String, Serializable>> entry : matchingResources.entrySet()) {
@@ -171,7 +240,7 @@ public class HazelcastResourceDirectory extends DefaultResourceDirectory {
                 MultiMap<String, String> idMapping = getIDMapping();
                 Collection<String> concreteIds = idMapping.get(id.toString());
                 if (concreteIds != null && !concreteIds.isEmpty()) {
-                    IMap<String, Map<String,Serializable>> allResources = getResourceMapping();
+                    IMap<String, Map<String, Serializable>> allResources = getResourceMapping();
                     Map<String, Map<String, Serializable>> resources = allResources.getAll(new HashSet<String>(concreteIds));
                     if (resources != null) {
                         for (Entry<String, Map<String, Serializable>> entry : resources.entrySet()) {
@@ -189,6 +258,9 @@ public class HazelcastResourceDirectory extends DefaultResourceDirectory {
 
     @Override
     protected IDMap<Resource> doRemove(Collection<ID> ids) throws OXException {
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("Removing IDs from HazelcastResourceDirectory: " + ids);
+        }
         IDMap<Resource> removedResources = new IDMap<Resource>();
         Set<ID> generalIds = new HashSet<ID>();
         Set<ID> resourceIds = new HashSet<ID>();
@@ -234,15 +306,20 @@ public class HazelcastResourceDirectory extends DefaultResourceDirectory {
             tx.rollback();
             throw new OXException(t);
         }
-
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("Removed Resource(s) from HazelcastResourceDirectory: " + removedResources);
+        }
         return removedResources;
     }
 
     @Override
     protected IDMap<Resource> doRemove(ID id) throws OXException {
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("Removing ID from HazelcastResourceDirectory: " + id);
+        }
         IDMap<Resource> removedResources = new IDMap<Resource>();
         MultiMap<String, String> idMapping = getIDMapping();
-        IMap<String, Map<String,Serializable>> allResources = getResourceMapping();
+        IMap<String, Map<String, Serializable>> allResources = getResourceMapping();
         if (id.isGeneralForm()) {
             Transaction tx = newTransaction();
             tx.begin();
@@ -279,7 +356,9 @@ public class HazelcastResourceDirectory extends DefaultResourceDirectory {
                 throw new OXException(t);
             }
         }
-
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("Removed Resource(s) from HazelcastResourceDirectory: " + removedResources);
+        }
         return removedResources;
     }
 
@@ -290,9 +369,11 @@ public class HazelcastResourceDirectory extends DefaultResourceDirectory {
     @Override
     protected HazelcastResource doSet(ID id, Resource resource, boolean overwrite) throws OXException {
         HazelcastResource hazelcastResource = new HazelcastResource(resource);
+        id.on(ID.Events.REFRESH, TOUCH_ID);
+        id.on(ID.Events.DISPOSE, CLEAN_UP);
 
         MultiMap<String, String> idMapping = getIDMapping();
-        IMap<String, Map<String,Serializable>> allResources = getResourceMapping();
+        IMap<String, Map<String, Serializable>> allResources = getResourceMapping();
         HazelcastResource previousResource = null;
         Transaction tx = newTransaction();
         if (LOG.isDebugEnabled()) {
@@ -305,7 +386,7 @@ public class HazelcastResourceDirectory extends DefaultResourceDirectory {
             // don't overwrite exisiting Presence Data
             if (hazelcastResource.getPresence() == null) { // a DefaultResource / idle reconnect
                 Map<String, Serializable> previousResourceMap = allResources.get(id.toString());
-                if(previousResourceMap != null) {
+                if (previousResourceMap != null) {
                     previousResource = HazelcastResourceWrapper.unwrap(previousResourceMap);
                 }
                 if (previousResource != null && previousResource.getPresence() != null) {
@@ -315,7 +396,7 @@ public class HazelcastResourceDirectory extends DefaultResourceDirectory {
                     if (overwrite) {
                         previousResourceMap = allResources.put(id.toString(), HazelcastResourceWrapper.wrap(hazelcastResource));
                         previousResource = HazelcastResourceWrapper.unwrap(previousResourceMap);
-                    } else {    
+                    } else {
                         previousResourceMap = allResources.putIfAbsent(id.toString(), HazelcastResourceWrapper.wrap(hazelcastResource));
                         previousResource = HazelcastResourceWrapper.unwrap(previousResourceMap);
                     }
@@ -356,23 +437,12 @@ public class HazelcastResourceDirectory extends DefaultResourceDirectory {
             if (candidatePresence == null || candidatePresence.getPriority() < 0) {
                 continue;
             }
-            if (selectedResource == null) {
+            if ((selectedResource == null) || (selectedResource.getTimestamp().compareTo(candidateResource.getTimestamp()) < 0)) {
                 selectedResource = candidateResource;
-                continue;
-            } else {
-                int comparisonResult = selectedResource.getTimestamp().compareTo(candidateResource.getTimestamp());
-                if (comparisonResult < 0) {
-                    selectedResource = candidateResource;
-                }
             }
         }
 
-        if (selectedResource == null) {
-            return null;
-        } else {
-            return selectedResource.getPresence();
-        }
-
+        return selectedResource == null ? null : selectedResource.getPresence();
     }
 
     /**
@@ -383,17 +453,16 @@ public class HazelcastResourceDirectory extends DefaultResourceDirectory {
      * @throws OXException
      */
     private HazelcastResource conjureResource(ID id) throws OXException {
-        if (conjure(id)) {
-            HazelcastResource res = new HazelcastResource();
-            HazelcastResource meantime = setIfAbsent(id, res);
-            id.on(ID.Events.DISPOSE, CLEAN_UP);
-            if (meantime == null) {
-                return res;
-            }
-            return meantime;
-        } else {
+        if (!conjure(id)) {
             return null;
         }
+        syntheticIDs.add(id);
+        HazelcastResource res = new HazelcastResource();
+        HazelcastResource meantime = setIfAbsent(id, res);
+        if (meantime == null) {
+            return res;
+        }
+        return meantime;
     }
 
     /*
@@ -409,14 +478,14 @@ public class HazelcastResourceDirectory extends DefaultResourceDirectory {
     }
 
     /**
-     * Get the mapping of general IDs to full IDs e.g. marc.arens@premium <-> ox://marc.arens@premuim/random.
+     * Get the mapping of general IDs to full IDs e.g. marc.arens@premium <-> ox://marc.arens@premium/random.
      * 
      * @return the map used for mapping general IDs to full IDs.
-     * @throws OXException if the HazelcastInstance is missing. 
+     * @throws OXException if the HazelcastInstance is missing.
      */
-    protected MultiMap<String, String> getIDMapping() throws OXException {
-            HazelcastInstance hazelcast = HazelcastAccess.getHazelcastInstance();
-            return hazelcast.getMultiMap(id_map);
+    public MultiMap<String, String> getIDMapping() throws OXException {
+        HazelcastInstance hazelcast = HazelcastAccess.getHazelcastInstance();
+        return hazelcast.getMultiMap(id_map);
     }
 
     /**
@@ -425,7 +494,7 @@ public class HazelcastResourceDirectory extends DefaultResourceDirectory {
      * @return the map used for mapping full IDs to ResourceMaps.
      * @throws OXException if the map couldn't be fetched from hazelcast
      */
-    protected IMap<String, Map<String, Serializable>> getResourceMapping() throws OXException {
+    public IMap<String, Map<String, Serializable>> getResourceMapping() throws OXException {
         HazelcastInstance hazelcast = HazelcastAccess.getHazelcastInstance();
         return hazelcast.getMap(resource_map);
     }
@@ -438,7 +507,7 @@ public class HazelcastResourceDirectory extends DefaultResourceDirectory {
 
             @Override
             public void rollback() throws IllegalStateException {
-
+                /* nothing */
             }
 
             @Override
@@ -448,23 +517,45 @@ public class HazelcastResourceDirectory extends DefaultResourceDirectory {
 
             @Override
             public void commit() throws IllegalStateException {
-
+                /* nothing */
             }
 
             @Override
             public void begin() throws IllegalStateException {
-
+                /* nothing */
             }
         };
     }
 
+    /**
+     * Starts the timer that refreshes synthetic resources
+     */
+    protected void startRefreshTimer() {
+        Services.getService(TimerService.class).scheduleAtFixedRate(new Runnable() {
+
+            @Override
+            public void run() {
+                for (ID syntheticID : syntheticIDs) {
+                    syntheticID.trigger(ID.Events.REFRESH, this);
+                }
+            }
+
+        }, 1, 15, TimeUnit.MINUTES);
+    }
+
+    /**
+     * Remove the entries we track for this id from the IDMapping and the ResourceMapping
+     */
     private final IDEventHandler CLEAN_UP = new IDEventHandler() {
 
         @Override
         public void handle(String event, ID id, Object source, Map<String, Object> properties) {
             if (source != HazelcastResourceDirectory.this) {
                 try {
-                    removeWithoutDisposeEvent(id);
+                    IDMap<Resource> removed = removeWithoutDisposeEvent(id);
+                    if (LOG.isDebugEnabled()) {
+                        LOG.debug("Removed: " + removed.entrySet().toString());
+                    }
                 } catch (OXException e) {
                     LOG.error(e.getMessage(), e);
                 }
@@ -473,4 +564,39 @@ public class HazelcastResourceDirectory extends DefaultResourceDirectory {
 
     };
 
+    /**
+     * Touch the infos we track for a given ID so they don't get automatically removed by Hazelcast's eviction policy as long as it's in
+     * active use.
+     */
+    private final IDEventHandler TOUCH_ID = new IDEventHandler() {
+
+        @Override
+        public void handle(String event, ID id, Object source, Map<String, Object> properties) {
+            try {
+                /*
+                 * This performs a set on map entries to prevent eviction
+                 */
+                getIDMapping().get(id.toGeneralForm().toString());
+                Map<String, Serializable> resourceWrap = getResourceMapping().get(id.toString());
+                if (resourceWrap == null) {
+                    if (LOG.isDebugEnabled()) {
+                        LOG.debug("Unable to refresh id, might have been removed in the meantime: " + id);
+                    }
+                    syntheticIDs.remove(id);
+                } else {
+                    resourceWrap.put(HazelcastResourceWrapper.eviction_timestamp, System.currentTimeMillis());
+                    Map<String, Serializable> put = getResourceMapping().put(id.toString(), resourceWrap);
+                    if (put == null) {
+                        LOG.warn("There was no previous entry associated with id: " + id + "when refreshing the directory entry via write.");
+                    } else {
+                        LOG.debug("Refreshed id: " + id.toString() + " with resource: " + resourceWrap);
+                    }
+                }
+            } catch (OXException e) {
+                LOG.error(e.getMessage());
+            }
+        }
+
+    };
+
 }
