@@ -51,6 +51,7 @@ package com.openexchange.realtime;
 
 import javax.annotation.concurrent.NotThreadSafe;
 import com.openexchange.exception.OXException;
+import com.openexchange.realtime.packet.ID;
 import com.openexchange.realtime.packet.Stanza;
 
 
@@ -72,5 +73,11 @@ public @NotThreadSafe interface ComponentHandle {
      * true if the resource directory state might be changed by this method call
      */
     boolean shouldBeDoneInGlobalThread(Stanza stanza);
+    
+    /**
+     * Get the id used to direct messages at this {@link ComponentHandle}
+     * @return the id used to direct messages at this {@link ComponentHandle}
+     */
+    ID getID();
 
 }
